import yarp
import pyicub
from pyicub.iCubHelper import iCub, iCubRequest, iCubTask, ICUB_PARTS, JointAction
pyicub.controllers.PositionController.MIN_JOINTS_DIST = 20

TIMEOUT_LOOKAT = 0.4
TIMEOUT_LANDMARKS = 0.5
TIMEOUT_EYELIDS = 0.5

import time

class iCubHelper:

    def __init__(self):
        self.icub = iCub(configuration_file="/home/docky/shared/exp/icub.yaml", disable_logs=True)
        self.icub.gaze.reset()
        self.icub.gaze.setTrackingMode(True)
        self.icub.emo.smile()
        a = JointAction(ICUB_PARTS.LEFT_ARM, req_time=2.0, target_position=[0.0, 15.0, 0.0, 25.0, 0.0, 0.0, 0.0, 60.0, 20.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0])
        b = JointAction(ICUB_PARTS.RIGHT_ARM, req_time=2.0, target_position=[0.0, 15.0, 0.0, 25.0, 0.0, 0.0, 0.0, 60.0, 20.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0])
        self.icub.move(a)
        self.icub.move(b)

    @iCubTask.request(timeout=TIMEOUT_LOOKAT)
    def lookMono(self, cam, x, y, z):
        p = yarp.Vector(2)
        p.set(0, float(x))
        p.set(1, float(y))
        self.icub.gaze.IGazeControl.lookAtMonoPixel(cam, p, z)
        yarp.delay(0.3)

    @iCubTask.request(timeout=TIMEOUT_LOOKAT)
    def lookAtAbsAngles(self, azi, ele, ver):
        self.icub.gaze.lookAtAbsAngles(azi, ele, ver)

    def initGaze(self):
        self.icub.gaze.reset()
        self.icub.gaze.IGazeControl.setNeckTrajTime(0.4)
        self.icub.gaze.IGazeControl.setEyesTrajTime(0.2)
        self.icub.gaze.IGazeControl.blockEyes(3.5)
        self.icub.gaze.setTrackingMode(True)


    def lookStraight(self, _wait = True):
        req = self.lookAtAbsAngles(0.0, 0.0, 3.5)
        if _wait is True:
            req.wait_for_completed()
        return req

    def lookDown(self, _wait = True):
        req = self.lookAtAbsAngles(0.0, -24.0, 3.5)
        if _wait is True:
            req.wait_for_completed()
        return req

    def lookRight(self):
        req = self.lookAtAbsAngles(-27.0, -12.0, 3.5)
        self.icub.gaze.waitMotionOnset(speed_ref=50)
        return req

    def lookLeft(self):
        req = self.lookAtAbsAngles(27.0, -12.0, 3.5)
        self.icub.gaze.waitMotionOnset(speed_ref=50)
        return req

    @iCubTask.request(timeout=TIMEOUT_LOOKAT)
    def lookRest(self, _wait = True):
        self.lookAtAbsAngles(0.0, -12.0, 3.5)

    @iCubTask.request(timeout=TIMEOUT_LOOKAT)
    def lookAtPPT(self, landmarks):
        if landmarks is None or landmarks == (None, None):
            self.lookStraight()
            return 0
        else:
            self.lookMono(1, int(landmarks[0]/2)-20, int(landmarks[1]/2), 0.8).wait_for_completed()
            return 1

    def getAngles(self):
        angles = yarp.Vector(3)
        self.icub.gaze.IGazeControl.getAngles(angles)
        return [angles[0], angles[1], angles[2]]

    @iCubTask.request(timeout=TIMEOUT_EYELIDS)
    def CloseEyes(self):
        self.icub.emo.closingEyes()

    @iCubTask.request(timeout=TIMEOUT_EYELIDS)
    def OpenEyes(self):
        self.icub.emo.openingEyes()

    def close(self):
        self.lookStraight()
